/*
*  C Implementation: shvar
*
* Description: Screen handler variables
*
*
* Author: Ricardo Cantu <ricky@smartcsc.com>, (C) 2006
*
* Copyright: See COPYING file that comes with this distribution
*
*/

#include     <stdlib.h>
#include     <signal.h>
#include     <string.h>
#include     <errno.h>
#include     <fcntl.h>
#include     <locale.h>
#include     <sys/types.h>
#include     <unistd.h>
#include     <sys/msg.h>
#include     <ctype.h>

#include     <curses.h>
#include     <form.h>
#include     <cdk/cdk.h>
#include     <fcgi_config.h>
#include     <fcgiapp.h>
#include     <bstrlib.h>

#include     "../uc/eng_types.h"
#include     "../uc/uc.h"
#include     "../uc/eng_utf8.h"
#include     "../cp/xtwa.c"
#include     "sh_proto.h"
#include     "sh.h"
#include     "../fh/fh.h"
#include     "../ph/ph.h" 


#define SH_NUM_SCREEN      4         /* Number of distinct screen storage */
#define SH_NUM_WINDOW      4         /* Number of distinct windows inside screens */
#define SH_TEXT            0x31      /* Screen mode 1 */
#define SH_WEB             0x32      /* Screen mode 2 */
#define SH_UNPRO_TYPE      1         /* Setting color mode */
#define SH_PRO_TYPE        2         /* Setting color mode */
#define SH_TMESG_LEN       132       /* Terminal message length */
#define SH_PUBLIC(i)       (((unsigned)((i)->opts) & (O_PUBLIC))==O_PUBLIC)
#define SH_ACTIVE(i)       (((unsigned)((i)->opts) & (O_ACTIVE))==O_ACTIVE)

#define SH_TXMP    0x504d2454          /* name of alternate TEMP file */
#define SH_TEMP    0x504d4554          /* define temp file name */
#define SH_2BLANKS 0x2020

/* Function keys returned to application */
/* F1-F9 = F1-F9, F10-F12 = 10-12 */
#define F1      0x3146
#define F2      0x3246
#define F3      0x3346
#define F4      0x3446
#define F5      0x3546
#define F6      0x3646
#define F7      0x3746
#define F8      0x3846
#define F9      0x3946
#define F10     0x3031
#define F11     0x3131
#define F12     0x3231
#define F13     0x3331
#define F14     0x3431
#define F15     0x3531
#define F16     0x3631
#define F17     0x3731
#define F18     0x3831
#define F19     0x3931
#define F20     0x3032
#define F21     0x3132
#define F22     0x3232
#define F23     0x3332
#define F24     0x3432

#define DEFAULT_UPRO    0x20
#define DEFAULT_PRO     0xa0
#define NONDISP_UPRO    0x50
#define NONDISP_PRO     0x70

#define COLOR_UPRO_BEG  0x40
#define COLOR_UPRO_END  0x51  /* q */

#define COLOR_PRO_BEG   0x60
#define COLOR_PRO_END   0x71  /* Q */

int SH_HTML_ROW_PIXEL;
int SH_HTML_COL_PIXEL;
int SH_HTML_FONT_NAME;
int SH_HTML_FONT_SIZE;

/* Colors */
char SH_FORE_BLACK = '@';
char SH_FORE_BLUE = 'A';
char SH_FORE_BLINK_YELLOW = 'B';
char SH_FORE_CYAN = 'C';
char SH_FORE_RED = 'D';
char SH_FORE_MAGENTA = 'E';
char SH_FORE_BROWN = 'F';
char SH_FORE_BEIGE = 'G';
char SH_FORE_GRAY = 'H';
char SH_FORE_LIGHT_BLUE = 'I';
char SH_FORE_LIGHT_GREEN = 'J';
char SH_FORE_LIGHT_CYAN = 'K';
char SH_FORE_LIGHT_RED = 'L';
char SH_FORE_LIGHT_MAGENTA = 'M';
char SH_FORE_YELLOW = 'N';
char SH_FORE_HIGH_WHITE = 'O';
char SH_FORE_WHITESMOKE = 'Q';

char SH_BACK_BLACK = '@';
char SH_BACK_BLUE = 'A';
char SH_BACK_BLINK_YELLOW = 'B';
char SH_BACK_CYAN = 'C';
char SH_BACK_RED = 'D';
char SH_BACK_MAGENTA = 'E';
char SH_BACK_BROWN = 'F';
char SH_BACK_GREY = 'G';
char SH_BACK_WHITE = 'O';
char SH_BACK_WHITESMOKE = 'Q';

/* Field types */
char SH_TYPE_RADIO = '0';
char SH_TYPE_CHECK = '1';
char SH_TYPE_BUTTON = '2';
char SH_TYPE_LIST = '3';
char SH_TYPE_LINK = '4';
char SH_TYPE_FLST = '5';              /* Select list with data from a text file */

/* Key mapping literals */
char            F1_map[] = "F1";
char            F2_map[] = "F2";
char            F3_map[] = "F3";
char            F4_map[] = "F4";
char            F5_map[] = "F5";
char            F6_map[] = "F6";
char            F7_map[] = "F7";
char            F8_map[] = "F8";
char            F9_map[] = "F9";
char            F10_map[] = "F10";
char            F11_map[] = "F11";
char            F12_map[] = "F12";
char            F13_map[] = "F13";
char            F14_map[] = "F14";
char            F15_map[] = "F15";
char            F16_map[] = "F16";
char            F17_map[] = "F17";
char            F18_map[] = "F18";
char            F19_map[] = "F19";
char            F20_map[] = "F20";
char            F21_map[] = "F21";
char            F22_map[] = "F22";
char            F23_map[] = "F23";
char            F24_map[] = "F24";
char            ENT_map[] = "ENT";
char            HOME_map[] = "HOME";
char            END_map[] = "END";
char            DOWN_map[] = "DOWN";
char            UP_map[] = "UP";
char            PGUP_map[] = "PGUP";
char            PGDWN_map[] = "PGDWN";
char            TAB_map[] = "TAB";
char            ESC_map[] = "ESC";
char            BACKSPACE_map[] = "BACKSPACE";


struct
{
  char            init;             /* screen initialization complete */
  char            screen_mode;      /* Text or Web */
  char            filter_pad;       /* filter trailing pad char from user data */
  char            pad_top_line;     /* Use pad char on top line */
  char            hide_top_line;    /* Do not show top line  */
  char            first_key_blank;  /* Auto blank on first keystroke */
  char            pad_reverse;      /* Show unprotected in reverse video */
  char            detached;         /* Log screen activity only */
  char            log;              /* Log function */
} Sh_flags =
{0,0,0,0,0,0,0,0,0};

/* all possable colors */
struct _color
{
  unsigned char   fore;         /* Engine attrib */
  char           *html_fore;    /* html translated name*/
  unsigned char   back;
  char           *html_back;
  int             attrib;
};

/* Save protected data */
struct _savp
{
  short int       tmplin;       /* protected line # */
  short int       tmpcol;       /* protected column # */
  short int       tmplen;       /* protected length */
  char            tmpdat[20];   /* protected data value */
  unsigned char   tmpatr;       /* protected field attribute */
  unsigned char   tmpsty;       /* Protected style */
  unsigned char   tmpfnt;       /* Protected font */
  char            tmpfsz[3];    /* Protected size */
  struct _savp   *tmpnxt;       /* point to next */
};

struct _fcgi_header
{
  char            display_type; /* 0=Reg scr, 2=popup scr 3=mesg scr*/
  char            screen_name[8];
};

char            templist[] = "LINE=02,ULINUPOSULENPLINPPOSPLENPCAPPATRUATRURLNPFNTUFNTPFSZUFSZPSTYUSTYUTYPTCAPWCAPBCAPBVALBKEY****RLSE";
char            templist3[] = "LINE=03,LIST****RLSE";

struct _temp02
{
  char            tmulin[2];    /* unprotected line # */
  char            tmucol[2];    /* unprotected column # */
  char            tmulen[2];    /* unprotected length */
  char            tmplin[2];    /* protected line # */
  char            tmpcol[2];    /* protected column # */
  char            tmplen[2];    /* protected length */
  char            tmpdat[20];   /* protected data value */
  unsigned char   tmpatr;       /* protected field attribute */
  unsigned char   tmuatr;       /* unprotected field attribute */
  char            tmurln[3];    /* unprotected real length */
  unsigned char   tmpfnt;       /* pnprotected font */
  unsigned char   tmufnt;       /* unprotected font */
  char            tmpfsz[2];    /* pnprotected size */
  char            tmufsz[2];    /* unprotected size */
  unsigned char   tmpsty;       /* unprotected style */
  unsigned char   tmusty;       /* unprotected style */
  unsigned char   tmutyp;       /* unprotected field type */
  char            tmtcap[80];   /* Terminal caption */
  char            tmwcap[80];   /* Web Caption */
  char            tmbcap[25];   /* button Caption */
  char            tmbval[20];   /* button value */
  char            tmbkey[2];    /* button key */
};

struct _temp03
{
  char    list[100];
};

struct _sh_fh
{
  struct _fh_parm hd;
  char            fhkv[8];
  char            fhln[sizeof (templist) - 1];          /* LINE=0?,????****RLSE */
  char            data[sizeof (struct _temp02)];    /* data */
};

struct _sh_fh_3
{
  struct _fh_parm hd;
  char            fhkv[8];
  char            fhkv_row_col[4];
  char            fhln[sizeof (templist3) - 1];        /* LINE=0?,????****RLSE */
  char            data[sizeof (struct _temp03)];   /* data */
};

struct _tmhdr                   /* template header record */
{
  unsigned char   hdbatr;       /* screen background attribute */
  unsigned char   hdfil1;       /* Filler */
  char            hdlins[2];    /* number of screen lines */
  char            hdcols[2];    /* screen width */
  char            hdtlin[2];    /* screen line for top-left corner */
  char            hdlcol[2];    /* screen col for top-left corner */
  unsigned char   hdlin1;       /* Hide line 1 */
  unsigned char   hdbord;       /* Border around screen 1=single 2=double*/
};

struct _sh_screen
{
  struct _tmhdr   temp_header;                 /* 0=line header on TEMP */
  char           *input_buffer;                /* input buffer */
  int             input_buffer_length;         /* input buffer length */
  char            line1_data[ENG_SH_L1LNTH];   /* Save line 1 data */
  char            web_cursor[4];               /* Return cursor from web ui */
  FIELD          *field[255];                  /* 255 available fields */
  FORM           *form;         /* Current form */
  WINDOW         *current_win;  /* Window */
  short int       num_field;    /* Current number of fields on screen */
  short int       beg_usr_data; /* Where data begins in user's request */
  short int       cur_row;      /* Current row */
  short int       cur_col;      /* Current col */
  int             focus_field;  /* Field to put curser on */
  short int       line_adj;     /* For hide top line */
  char            is_popup;     /* Is this a popup window */
  struct _savp   *sav_pdat_beg; /* pointer to beginging of saved protected fields data */
  struct _savp   *sav_pdat;     /* pointer to saved protected fields data */
};

struct _sh_windows
{
  struct _sh_screen  *window[SH_NUM_WINDOW];  /* Allow only so many deep */
  short int           cur_window;             /* Current working window */
};

struct _sh_field_info
{
  unsigned char   org_att;      /* Original Attribute */
  unsigned char   ufnt;         /* unprotected font */
  char            ufsz[3];      /* unprotected size */
  unsigned char   usty;         /* unprotected style */
  unsigned char   utyp;         /* Field type */
  char            tcap[81];     /* Terminal caption */
  char            wcap[81];     /* Web caption */
  char            bcap[26];     /* button caption */
  char            bval[21];     /* button value */
  char            bkey[3];      /* button key */
  short           cur_col;      /* Current column */
};

struct _sh_windows  Sh_scr_data[SH_NUM_SCREEN];   /* Actual storage space for the screen struc */
struct _sh_screen  *Sh_scr_ptr;                   /* Pointer to the current screen sruct */
short int           Sh_current_scr_core;          /* Index to screen core */

struct _tmhdr      *Sh_temp_header;               /* 0=line header on TEMP */
struct _sh_parm    *Sh_sh;                        /* pointer to user's call area */
char                Sh_prev_screen[8];            /* The previous screen name */
struct sigaction    Sh_sa;                        /* Signal struct */
int                 Sh_mesg_id;                   /* message identifier */
char                Sh_mesg1[SH_TMESG_LEN];       /* Status message 1 */
char                Sh_mesg2[SH_TMESG_LEN];       /* Status message 2 */
WINDOW             *Sh_mesg_win;                  /* Window used to display mesg*/
FILE               *Sh_log_fd;                    /* file pointer for log file */
int                 Sh_mast_pid;                  /* Master process pid */
char                Sh_eng_id[3];                 /* Engine id */
struct _sh_fh       Sh_fh;                        /* File handle for temp table */
struct _temp02     *Sh_temp02;                   /* TEMP line=01 */
struct _sh_fh_3     Sh_fh_3;                      /* File handle for temp table */
struct _temp03     *Sh_temp03;                    /* TEMP line=03 */
struct _color       Sh_color_pair[17];            /* Used to define curses color pairs */
union rint          Sh_key_pressed;               /* Key Pressed */
struct _eng_config_keys_list *Sh_config_keys;      /* Key mappings */
struct _eng_config_keys_list *Sh_beg_config_keys;  /* Pointer to first one */
unsigned int        Sh_ins_toggle;                 /* Global insert state */

unsigned char       Sh_def_sys_background;        /* Default background color */
unsigned char       Sh_def_sys_protected;         /* Default protected color */
unsigned char       Sh_def_sys_unprotected;       /* Default protected color */
unsigned char       Sh_def_background;            /* Default background color */
unsigned char       Sh_def_protected;             /* Default protected color */
unsigned char       Sh_def_unprotected;           /* Default unprotected color */
unsigned char       Sh_def_pad_char;              /* default unpro field padding */

FCGX_Request        Sh_fcgi_io;                   /* FCGI Request variable.  Handles inputs outputs and error */
bstring             Sh_html_buffer;               /* Regular data html buffer */
bstring             Sh_html_pro_buffer;           /* Protect data html buffer */
bstring             Sh_html_unpro_buffer;         /* Protect data html buffer */

/* html work fields */
bstring             Sh_focus_name;
bstring             Sh_div_dragable;
bstring             Sh_style_position;
bstring             Sh_style_dimensions;
bstring             Sh_style_color;
bstring             Sh_style_button;
bstring             Sh_style_decoration;
bstring             Sh_style_font;
bstring             Sh_input_focus;
bstring             Sh_mouse_over_out;
bstring             Sh_field_name_id;
bstring             Sh_field_html_size;
bstring             Sh_form_type;
struct _fcgi_header Sh_fcgi_header;               /* Header for fcgi */
ENG_FLAG            Sh_wrote_html;              /* Page already written so data is in the buffer */
char               *Sh_html_color;

struct tagbstring   Sh_html_form_beg = bsStatic ("\n<form method=\"post\" name=\"main\" action=\"#\" style=\"border-style: none; border: 0; margin: 0; padding: 0;\">");
struct tagbstring   Sh_div_drag = bsStatic ("onmousedown=\"sh_init_drag(event)\" onmouseup=\"sh_stop_drag()\"");
struct tagbstring   Sh_style_unders = bsStatic ("border-style: solid; border-bottom-width: thin; border-color: white;");
struct tagbstring   Sh_style_border_box = bsStatic ("border-style: groove; border-bottom: none; border-right: none;");
struct tagbstring   Sh_html_header = bsStatic ("Content-type: text/html; charset=utf-8\r\n\r\n");

char               *Sh_html_colors[] = {
      "Blue",         /* 0 */
      "Yellow",       /* 1 */
      "Cyan",         /* 2 */
      "Red",          /* 3 */
      "Magenta",      /* 4 */
      "Brown",        /* 5 */
      "Beige",        /* 6 */
      "DarkGray",     /* 7 */
      "LightBlue",    /* 8 */
      "LightGreen",   /* 9 */
      "LightCyan",    /* 10 */
      "Crimson",      /* 11 */
      "MediumOrchid", /* 12 */
      "White",        /* 13 */
      "Black",        /* 14 */
      "#F0F0F0"       /* 15 whitesmoke*/
    };
enum { SH_HTML_BLUE = 0, SH_HTML_YELLOW, SH_HTML_CYAN,
      SH_HTML_RED, SH_HTML_MAGENTA, SH_HTML_BROWN, SH_HTML_BEIGE,
      SH_HTML_GRAY, SH_HTML_LIGHT_BLUE, SH_HTML_LIGHT_GREEN,
      SH_HTML_LIGHT_CYAN, SH_HTML_LIGHT_RED, SH_HTML_LIGHT_MAGENTA,
      SH_HTML_WHITE, SH_HTML_BLACK, SH_HTML_WHITESMOKE};

char *Sh_html_fonts[] = {
  "LucidaTypewriter, monospace",   /* 0 */
  "Serif",              /* 1 */
  "Sans-serif",         /* 2 */
  "Courier"             /* 3 */
      };

enum { SH_HTML_STYLE_BOX = 1, SH_HTML_STYLE_ROUND};
